#!/usr/bin/python

# ________________________________________________________________
#
# PND launcher / installer script
# version 0.1.1
#
# This is intended to be associated with PND files and prompts the
# user to either run or install the PND file (w/ some options).
# When installing the pandora/apps, pandora/menu or pandora/desktop
# directories are created (if required) and the PND file is moved!
# to that directory.
#
# WARNINGS and todos:
# - directory creation is untested.
# - resizing the GUI looks ugly
# - default directories should be read from /etc/pandora/conf/desktop
#   instead of being hardcoded
# - add menu with explanation about pnd system and directories
# - add support for PND removal and appdata cleanup
# - handle moving between sd cards more carefully
#   - make sure it fits
#   - move over appdata as well
# ________________________________________________________________
#

import os
import sys
import pygtk
pygtk.require("2.0")
import gtk

# ----------------------------------------------------------------

# labels:
# application window title
LBL_TITLE = 'PND'
# options groupbox heading
LBL_OPTIONS = 'Install options'
# install option labels (underscores precede the hotkey (hit alt+key))
LBL_SD1 = 'SD_1'
LBL_SD2 = 'SD_2'
LBL_MENU = '_menu'
LBL_DTOP = '_desktop'
# button captions (underscores precede hotkey)
LBL_INST = '_Install'
LBL_RUN = '_Run'

# shell commands:
LAUNCH_CMD = 'pnd_run %s &'
MAKEDIR_CMD = 'mkdir -p %s'
INSTALL_CMD = 'mv %s %s'
MOUNTPOINT_CMD = "mount | grep /dev/mmcblk%ip | awk '{print $3}'"

def get_mountpoint(sd_card_nr):
  """Returns mountpoint of sd_card 'sd_card_nr: 0|1'."""
  return os.popen(MOUNTPOINT_CMD % sd_card_nr).read().strip()

# default installation paths:
# probably better to read /etc/pandora/conf/desktop for these instead
PATH_APPS = 'apps'
PATH_MENU = 'menu'
PATH_DTOP = 'desktop'
PATH_BASE = 'pandora'
PATH_SD1 = get_mountpoint(0)
PATH_SD2 = get_mountpoint(1)

# errors and warnings:
# error for not selecting any checkboxes
MSG_INVALID_DEST = 'You must choose %s, %s or both.' % (
  LBL_MENU.replace('_',''), LBL_DTOP.replace('_',''))
# error for failing to determine mountpoint of at least 1 sd-card
MSG_INVALID_SD = 'Could not determine SD-card path.'
# error for omitting the required application parameter
ERR_PARAMS = "This application requires the path to a PND file as single argument."
ERR_INVALID_PND = "Argument (%s) is not a file."

# SIGNALS
SIG_DESTROY = "destroy"
SIG_CLICKED = "clicked"

# ----------------------------------------------------------------

class PND_Installer(gtk.Window):
  def __init__(self, pndpath, *args, **kwargs):
    """PND launcher/management front-end application.

    This constructs a window containing a label, two radio buttons
    for selecting the SD card, two checkboxes for installing to
    menu and desktop respectively and two buttons for either
    installing (i.e. moving/copying) or running a PND file.

    @pndpath: string, fully qualified path to a PND file.
    """
    gtk.Window.__init__(self, *args, **kwargs)
    self.set_title(LBL_TITLE)

    self.pnd = pndpath

    self.vbox = gtk.VBox()
    self.add(self.vbox)

    self.label = gtk.Label()
    self.label.set_text(LBL_OPTIONS)

    self.vbox.add(self.label)
    
    self.sd1 = gtk.RadioButton(label=LBL_SD1)
    self.sd2 = gtk.RadioButton(group=self.sd1, label=LBL_SD2)
    self.menu = gtk.CheckButton(label=LBL_MENU)
    self.dtop = gtk.CheckButton(label=LBL_DTOP)
    self.menu.set_active(True)

    layout = ((self.sd1, self.sd2), (self.menu, self.dtop))
    self.options = gtk.Table(len(layout), max(len(i) for i in layout), True)
    self.vbox.add(self.options)

    for r, row in enumerate(layout):
      for c, item in enumerate(row):
        self.options.attach(item, c, c+1, r, r+1)

    self.buttons = gtk.HButtonBox()
    self.vbox.add(self.buttons)
    
    self.install = gtk.Button(label=LBL_INST)
    self.run = gtk.Button(label=LBL_RUN)

    self.buttons.add(self.install)
    self.buttons.add(self.run)

    self.connect(SIG_DESTROY, self.cmd_exit)
    self.run.connect(SIG_CLICKED, self.cmd_run, None)
    self.install.connect(SIG_CLICKED, self.cmd_install, None)

    self.show_all()

  def ShowError(self, err_msg):
    """Shows an error dialog containing the specified message.

    @err_msg: string, error to display
    """
    msg = gtk.MessageDialog(self, flags=gtk.DIALOG_MODAL,
                            type=gtk.MESSAGE_ERROR, 
                            #buttons=gtk.BUTTONS_CLOSE,
                             message_format=err_msg)
    msg.show()

  def cmd_run(self, widget, data):
    """launches the pnd specified from the commandline and exits.

    The PND is launched as a separate process and passed through pnd_run.
    """
    os.system(LAUNCH_CMD % self.pnd)
    self.cmd_exit()

  def cmd_install(self, widget, data):
    """Installs (i.e. moves) the pnd specified from the commandline.

    The file is moved to the proper location (apps/menu/desktop) and SD-card
    based on preferences flagged in the GUI.
    """
    sd1 = self.sd1.get_active()
    menu = self.menu.get_active()
    dtop = self.dtop.get_active()
    apps = menu and dtop
    if not (dtop or menu):
      self.ShowError(MSG_INVALID_DEST)
    elif not (PATH_SD1 or PATH_SD2):
      self.ShowError(MSG_INVALID_SD)
    else:
      mount = (PATH_SD1 or PATH_SD2) if sd1 else (PATH_SD2 or PATH_SD1)
      path = PATH_APPS if apps else (PATH_MENU if menu else PATH_DTOP)
      fullpath = os.path.join(mount, PATH_BASE, path)
      if not os.path.isdir(fullpath):
        # untested
        # os.system(MAKEDIR_CMD % fullpath)
        print MAKEDIR_CMD % fullpath
      os.system(INSTALL_CMD % (self.pnd, fullpath))
      print INSTALL_CMD % (self.pnd, fullpath)
      self.cmd_exit()

  def cmd_exit(self, widget=None, data=None):
    """Terminates the launcher application.

    If a PND was started, that application is left untouched.
    """
    gtk.main_quit()


def main(argv):
    if len(argv) != 2:
      raise Exception(ERR_PARAMS)
    elif not os.path.isfile(argv[1]):
      raise Exception(ERR_INVALID_PND % argv[1])
    else:
      app = PND_Installer(pndpath=argv[1])
      gtk.main()

if __name__ == "__main__":
    try:
        main(sys.argv)
    except Exception, e:
        sys.exit(e)
    else:
        sys.exit()
